Improve this doc View Source

The $provide service has a number of methods for registering components with the $injector. Many of these functions are also exposed on `angular.Module`.

An Angular service is a singleton object created by a service factory. These service factories are functions which, in turn, are created by a service provider. The service providers are constructor functions. When instantiated they must contain a property called `$get`, which holds the service factory function.

When you request a service, the $injector is responsible for finding the correct service provider, instantiating it and then calling its `$get` service factory function to get the instance of the service.

Often services have no configuration options and there is no need to add methods to the service provider. The provider will be no more than a constructor function with a `$get` property. For these cases the $provide service has additional helper methods to register services without specifying a provider.

- provider(provider) - registers a service provider with the $injector
- constant(obj) - registers a value/object that can be accessed by providers and services.
- value(obj) - registers a value/object that can only be accessed by services, not providers.
- factory(fn) - registers a service factory function, `fn`, that will be wrapped in a service provider object, whose `$get` property will contain the given factory function.
- service(class) - registers a constructor function, `class` that will be wrapped in a service provider object, whose `$get` property will instantiate a new object using the given constructor function.

See the individual methods for more information and examples.

## Methods

- ###

  `provider(name, provider);`

  Register a provider function with the $injector. Provider functions are constructor functions, whose instances are responsible for "providing" a factory for a service.

  Service provider names start with the name of the service they provide followed by `Provider`. For example, the $log service has a provider called $logProvider.

  Service provider objects can have additional methods which allow configuration of the provider and its service. Importantly, you can configure what kind of service is created by the `$get` method, or how that service will act. For example, the $logProvider has a method debugEnabled which lets you specify whether the $log service will log debug messages to the console or not.

  #### Parameters

  Param | Type | Details
  --- | --- | ---
  name | string | The name of the instance. NOTE: the provider will be available under `name + 'Provider'` key.
  provider | Objectfunction() | If the provider is: `Object`: then it should have a `$get` method. The `$get` method will be invoked using $injector.invoke() when an instance needs to be created. `Constructor`: a new instance of the provider will be created using $injector.instantiate(), then treated as `object`.

  #### Returns

  Object | registered provider instance

- ###

  `factory(name, $getFn);`

  Register a service factory, which will be called to return the service instance. This is short for registering a service where its provider consists of only a `$get` property, which is the given service factory function. You should use $provide.factory(getFn) if you do not need to configure your service in a provider.

  #### Parameters

  Param | Type | Details
  --- | --- | ---
  name | string | The name of the instance.
  $getFn | function() | The $getFn for the instance creation. Internally this is a short hand for `$provide.provider(name, {$get: $getFn})`.

  #### Returns

  Object | registered provider instance

- ###

  `service(name, constructor);`

  Register a service constructor, which will be invoked with `new` to create the service instance. This is short for registering a service where its provider's `$get` property is the service constructor function that will be used to instantiate the service instance.

  You should use $provide.service(class) if you define your service as a type/class.

  #### Parameters

  Param | Type | Details
  --- | --- | ---
  name | string | The name of the instance.
  constructor | Function | A class (constructor function) that will be instantiated.

  #### Returns

  Object | registered provider instance

- ###

  `value(name, value);`

  Register a value service with the $injector, such as a string, a number, an array, an object or a function. This is short for registering a service where its provider's `$get` property is a factory function that takes no arguments and returns the value service.

  Value services are similar to constant services, except that they cannot be injected into a module configuration function (see `config`) but they can be overridden by an Angular decorator.

  #### Parameters

  Param | Type | Details
  --- | --- | ---
  name | string | The name of the instance.
  value | * | The value.

  #### Returns

  Object | registered provider instance

- ###

  `constant(name, value);`

  Register a constant service, such as a string, a number, an array, an object or a function, with the $injector. Unlike value it can be injected into a module configuration function (see `config`) and it cannot be overridden by an Angular decorator.

  #### Parameters

  Param | Type | Details
  --- | --- | ---
  name | string | The name of the constant.
  value | * | The constant value.

  #### Returns

  Object | registered instance

- ###

  `decorator(name, decorator);`

  Register a service decorator with the $injector. A service decorator intercepts the creation of a service, allowing it to override or modify the behaviour of the service. The object returned by the decorator may be the original service, or a new service object which replaces or wraps and delegates to the original service.

  #### Parameters

  Param | Type | Details
  --- | --- | ---
  name | string | The name of the service to decorate.
  decorator | function() | This function will be invoked when the service needs to be instantiated and should return the decorated service instance. The function is called using the injector.invoke method and is therefore fully injectable. Local injection arguments: `$delegate` - The original service instance, which can be monkey patched, configured, decorated or delegated to.